require_relative 'setup'
class PaskTest < MiniTest::Test
  def setup
    @code = Code.new
  end

  class Code
    def call
      2+2
      x = 5
    end
  end

  def test_start
    tracer = Pask.new
    tracer.interested? { |e| e.return? }
    tracer.trace { @code.call }
    assert_instance_of Pask::Event, tracer.start
  end

  def test_start_without_a_match
    tracer = Pask.new
    tracer.interested? { false }
    tracer.trace { @code.call }
    assert_equal nil, tracer.start
  end

  def test_start_on_started_tracer
    tracer = Pask.new
    tracer.trace { @code.call }
    tracer.start
    assert_raises(Pask::NotFinishedError) { tracer.start }
  end

  def test_consume!
    tracer = Pask.new
    tracer.interested? { |e| e.call? or e.return? }
    tracer.trace { @code.call }
    events = tracer.consume!
    assert_equal 2, events.size
  end

  def test_sleeping_predicate_on_started_tracer
    tracer = Pask.new
    tracer.interested? { |event| event.return? }
    tracer.trace { @code.call }
    tracer.start
    assert_equal true, tracer.sleeping?
  end

  def test_sleeping_predicate_on_finished_tracer
    tracer = Pask.new
    tracer.interested? { |event| event.return? }
    tracer.trace { @code.call }
    tracer.start
    tracer.resume while tracer.resume
    assert_equal false, tracer.sleeping?
  end

  def test_started_predicate_on_started_tracer
    tracer = Pask.new
    tracer.interested? { |event| event.return? }
    tracer.trace { @code.call }
    tracer.start
    assert_equal true, tracer.started?
  end

  def test_started_predicate_on_unstarted_tracer
    tracer = Pask.new
    assert_equal false, tracer.started?
  end

  def test_resume_on_unstarted_tracer
    tracer = Pask.new
    assert_raises(Pask::NotStartedError) { tracer.resume }
  end

  def test_interested_predicate_without_block
    tracer = Pask.new
    assert_raises(ArgumentError) { tracer.interested? }
  end

  def test_interested_predicate_with_callable
    obj = Proc.new {}
    tracer = Pask.new
    assert_equal obj, tracer.interested?(obj)
  end

  def test_finished_predicate_on_finished_tracer
    tracer = Pask.new
    tracer.trace { @code.call }
    tracer.start
    tracer.resume while tracer.resume
    assert_equal true, tracer.finished?
  end

  def test_finished_predicate_on_unstarted_tracer
    tracer = Pask.new
    assert_equal false, tracer.finished?
  end

  def test_finished_predicate_on_started_tracer
    tracer = Pask.new
    tracer.interested? { |event| event.return? }
    tracer.trace { @code.call }
    tracer.start
    assert_equal false, tracer.finished?
  end

  def test_running_predicate_on_dead_tracer
    tracer = Pask.new
    tracer.interested? { |event| event.return? }
    tracer.trace { @code.call }
    tracer.start
    tracer.resume
    assert_equal false, tracer.running?
  end

  def test_running_predicate_on_unstarted_tracer
    tracer = Pask.new
    assert_equal false, tracer.running?
  end

  def test_concurrent_trace_with_binding_and_local_variable
    tracer = Pask.new
    tracer.interested? { |e| e.call? or e.return? }
    tracer.trace { @code.call }
    event = tracer.start
    assert_equal nil, event.binding.eval("x")
    event = tracer.resume
    assert_equal 5, event.binding.eval("x")
  end
end
